# Role `sane_debian_system` &ndash; set up a manageable Debian system

This role sets up a Debian system so that it can be managed with
Ansible in a reasonable way.

## Version history

### Version 2

* `sane_debian_hostname` defaults to the inventory hostname. This
  means it's not necessary to set it if the default is sufficient.

## Minimally sane Debian system

~~~scenario
given a host running Debian
when I use role sane_debian_system
and I use variables from sane1.yml
and I run the playbook
then the host has the sudo package installed
and the host has the apt-transport-https package installed
and the host has the locales package installed
and the host has the ntp package installed
and the host has an empty /etc/apt/sources.list.d directory
and the host has hostname saneone
~~~

~~~{#sane1.yml .file .yaml}
ansible_python_interpreter: /usr/bin/python3

sane_debian_system_version: 2
sane_debian_system_codename: buster
sane_debian_system_hostname: saneone
~~~

## Uses inventory hostname by default

~~~scenario
given a host running Debian
when I use role sane_debian_system
and I use variables from sane-without-hostname.yml
and I run the playbook
then the host has the sudo package installed
and the host has hostname debian-ansible-test
~~~

~~~{#sane-without-hostname.yml .file .yaml}
sane_debian_system_version: 2

sane_debian_system_codename: buster
~~~

## Checks that debian codename is set

~~~scenario
given a host running Debian
when I use role sane_debian_system
and I use variables from sane2.yml
and I try to run the playbook
then the command fails
and stdout contains "sane_debian_system_codename"
~~~

~~~{#sane2.yml .file .yaml}
sane_debian_system_version: 2
~~~
